package cn.fw.valhalla.component;

import cn.fw.member.sdk.event.PhoneChangeEvent;
import cn.fw.valhalla.domain.db.customer.Customer;
import cn.fw.valhalla.domain.db.customer.CustomerBaseInfo;
import cn.fw.valhalla.domain.enums.DefeatReasonEnum;
import cn.fw.valhalla.service.data.CustomerBaseInfoService;
import cn.fw.valhalla.service.data.CustomerService;
import cn.fw.valhalla.service.data.PublicCluePoolService;
import cn.fw.valhalla.service.data.StammkundePoolService;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import lombok.extern.slf4j.Slf4j;
import org.apache.rocketmq.spring.annotation.RocketMQMessageListener;
import org.apache.rocketmq.spring.core.RocketMQListener;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * @author : kurisu
 * @className : MemberPhoneChangeConsumer
 * @description : 会员手机号变更
 * @date: 2022-03-10 09:41
 */

@Slf4j
@Component
@RocketMQMessageListener(topic = PhoneChangeEvent.TOPIC, consumerGroup = "${spring.application.name}" + "-" + PhoneChangeEvent.TOPIC)
public class MemberPhoneChangeConsumer implements RocketMQListener<PhoneChangeEvent> {


    private final CustomerService customerService;
    private final CustomerBaseInfoService customerBaseInfoService;
    private final StammkundePoolService stammkundePoolService;
    protected PublicCluePoolService publicCluePoolService;

    @Autowired
    public MemberPhoneChangeConsumer(final CustomerService customerService,
                                     final CustomerBaseInfoService customerBaseInfoService,
                                     final StammkundePoolService stammkundePoolService) {
        this.customerService = customerService;
        this.customerBaseInfoService = customerBaseInfoService;
        this.stammkundePoolService = stammkundePoolService;
    }


    @Override
    public void onMessage(PhoneChangeEvent t) {
        final String str = "会员手机号变更mq";
        log.info("{}: MemberRemoveEvent=[{}]", str, t);
        try {
            // 会员号
            Long memberId = t.getMemberId();
            // 新手机号
            String phone = t.getNewPhone();
            // 移除的会员号
            Long removeUserId = t.getRemoveUserId();
            boolean isSame = Boolean.TRUE.equals(t.getSame());
            dealCustomer(memberId, phone, removeUserId, isSame);
        } catch (Exception ex) {
            log.error("{}失败，原因:{}", str, JSON.toJSONString(ex));
            throw ex;
        }
    }

    @Transactional(rollbackFor = Exception.class)
    public void dealCustomer(Long memberId, String phone, Long removeUserId, boolean same) {
        changeCustomerPhone(memberId, phone);
        if (Objects.isNull(removeUserId)) {
            return;
        }
        if (same) {
            mergeCustomer(memberId, removeUserId, phone);
        } else {
            forbiddenCustomer(removeUserId);
        }
    }

    /**
     * 变更手机号
     *
     * @param memberId
     * @param phone
     */
    private void changeCustomerPhone(Long memberId, String phone) {
        List<CustomerBaseInfo> list = customerBaseInfoService.list(Wrappers.<CustomerBaseInfo>lambdaQuery()
                .eq(CustomerBaseInfo::getMemberId, memberId)
                .eq(CustomerBaseInfo::getYn, Boolean.TRUE)
        );
        if (CollectionUtils.isEmpty(list)) {
            return;
        }
        list.forEach(r -> r.setMobile(phone));
        customerBaseInfoService.updateBatchById(list);
    }

    /**
     * 合并档案
     *
     * @param memberId
     * @param oldMemberId
     * @param phone
     */
    private void mergeCustomer(final Long memberId, final Long oldMemberId, String phone) {
        List<CustomerBaseInfo> list = customerBaseInfoService.list(Wrappers.<CustomerBaseInfo>lambdaQuery()
                .eq(CustomerBaseInfo::getMemberId, oldMemberId)
                .eq(CustomerBaseInfo::getYn, Boolean.TRUE)
        );
        if (CollectionUtils.isEmpty(list)) {
            return;
        }
        Map<Long, Long> groupBaseInfo = new HashMap<>();

        for (CustomerBaseInfo baseInfo : list) {
            List<Customer> customerList = customerService.queryByBaseId(baseInfo.getId());
            baseInfo.setYn(Boolean.FALSE);
            if (CollectionUtils.isEmpty(customerList)) {
                continue;
            }
            CustomerBaseInfo realBaseInfo = customerBaseInfoService.queryByMemberId(memberId, baseInfo.getGroupId());
            if (Objects.isNull(realBaseInfo)) {
                if (Objects.isNull(groupBaseInfo.putIfAbsent(baseInfo.getGroupId(), baseInfo.getId()))) {
                    baseInfo.setMemberId(memberId);
                    baseInfo.setMobile(phone);
                    baseInfo.setYn(Boolean.TRUE);
                }
            } else {
                groupBaseInfo.putIfAbsent(baseInfo.getGroupId(), realBaseInfo.getId());
            }

            for (Customer customer : customerList) {
                customer.setBaseId(groupBaseInfo.get(customer.getGroupId()));
            }
            customerService.updateBatchById(customerList);
        }
        customerBaseInfoService.updateBatchById(list);
    }


    /**
     * 禁用会员号对应档案
     *
     * @param memberId
     */
    private void forbiddenCustomer(Long memberId) {
        List<CustomerBaseInfo> list = customerBaseInfoService.list(Wrappers.<CustomerBaseInfo>lambdaQuery()
                .eq(CustomerBaseInfo::getMemberId, memberId)
                .eq(CustomerBaseInfo::getYn, Boolean.TRUE)
        );
        if (CollectionUtils.isEmpty(list)) {
            return;
        }
        List<Long> baseIdList = list.stream().map(CustomerBaseInfo::getId).collect(Collectors.toList());

        List<Customer> customerList = customerService.list(Wrappers.<Customer>lambdaQuery()
                .in(Customer::getBaseId, baseIdList)
                .eq(Customer::getYn, Boolean.TRUE));

        if (CollectionUtils.isEmpty(customerList)) {
            return;
        }
        for (Customer customer : customerList) {
            stammkundePoolService.reject(customer.getId(), customer.getGroupId(), DefeatReasonEnum.CI);
            publicCluePoolService.removeClue(customer.getId(), customer.getAdviserId());
            customerService.forbidden(customer.getId());
        }
    }
}
