package cn.fw.valhalla.service.bus.cust;

import cn.fw.common.data.mybatis.pagination.PageData;
import cn.fw.data.base.db.query.Page;
import cn.fw.valhalla.common.utils.*;
import cn.fw.valhalla.domain.db.MemberVehicle;
import cn.fw.valhalla.domain.db.customer.Customer;
import cn.fw.valhalla.domain.db.customer.CustomerBaseInfo;
import cn.fw.valhalla.domain.dto.NewVehicleDTO;
import cn.fw.valhalla.domain.query.MemberVehicleQueryVO;
import cn.fw.valhalla.domain.vo.IdVO;
import cn.fw.valhalla.domain.vo.customer.MemberVehicleVO;
import cn.fw.valhalla.rpc.member.MemberRpcService;
import cn.fw.valhalla.rpc.member.dto.MemberUserDTO;
import cn.fw.valhalla.sdk.enums.CarUseTypeEnum;
import cn.fw.valhalla.sdk.enums.CusTypeEnum;
import cn.fw.valhalla.service.data.*;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;

import static cn.fw.common.businessvalidator.Validator.V;

/**
 * 会员认证车辆
 * <p>
 * create at 2022-06-06
 *
 * @author xiatian
 */
@Slf4j
@Service
@RequiredArgsConstructor
public class MemberVehicleBizService {

    private final MemberVehicleService memberVehicleService;

    private final MemberRpcService memberRpcService;

    private final CustomerService customerService;

    private final CustomerBaseInfoService customerBaseInfoService;

    public List<MemberVehicleVO> getVehicleList(Long memberId) {
        List<MemberVehicle> memberVehicleList = memberVehicleService.list(Wrappers.<MemberVehicle>lambdaQuery()
                .eq(MemberVehicle::getYn, true)
                .eq(MemberVehicle::getMemberId, memberId));
        if (CollectionUtils.isEmpty(memberVehicleList)) {
            return Collections.emptyList();
        }

        return BeanUtils.copyList(memberVehicleList, MemberVehicleVO.class);
    }

    @Transactional(rollbackFor = Exception.class)
    public void saveVehicle(NewVehicleDTO dto) {
        V.notNull(dto,"车辆认证信息不能为空");
        //是否有认证车辆信息
        MemberVehicle memberVehicle = memberVehicleService.getVehicle(dto.getFrameNo());
        if (Objects.isNull(memberVehicle)) {
            saveNewVehicle(dto);
        } else {
            //校验是否重复添加
            memberVehicleService.update(Wrappers.<MemberVehicle>lambdaUpdate()
                    .set(MemberVehicle::getYn, false)
                    .eq(MemberVehicle::getFrameNo, dto.getFrameNo())
                    .eq(MemberVehicle::getYn, true));
            saveNewVehicle(dto);
        }
    }

    private void saveNewVehicle(NewVehicleDTO dto) {
        MemberUserDTO member = memberRpcService.user(dto.getMemberId());
        MemberVehicle newVehicle = BeanUtils.copy(dto, MemberVehicle.class);
        newVehicle.setMemberName(Objects.nonNull(member) ? member.getRealName() : null);
        newVehicle.setYn(Boolean.TRUE);
        newVehicle.setUseType(CarUseTypeEnum.ofValue(dto.getUseType()));
        newVehicle.setCusType(CusTypeEnum.ofValue(dto.getCusType()));
        memberVehicleService.save(newVehicle);

        //更新注册日期,车主信息
        List<Customer> customers = customerService.list(Wrappers.<Customer>lambdaQuery()
                .eq(Customer::getFrameNo, dto.getFrameNo()).eq(Customer::getYn, true));
        if (CollectionUtils.isEmpty(customers)) {
            return;
        }
        customers.forEach(customer -> {
            customer.setRegDate(dto.getRegDate());
            customer.setPlateNo(dto.getPlateNo());
            customer.setUseType(CarUseTypeEnum.ofValue(dto.getUseType()));
            if(Objects.isNull(customer.getSpecCode())){
                customer.setSpecCode(dto.getSpecCode());
            }
            CustomerBaseInfo customerBaseInfo = customerBaseInfoService.queryById(customer.getBaseId());
            if (Objects.nonNull(customerBaseInfo) && Objects.equals(dto.getMemberId(),customerBaseInfo.getMemberId()) &&
                   Objects.equals(CusTypeEnum.ofValue(dto.getCusType()),customerBaseInfo.getCusType())) {
                customerBaseInfo.setName(dto.getName());
                customerBaseInfo.setContact(dto.getName());
                customerBaseInfoService.updateById(customerBaseInfo);
            } else {
                CustomerBaseInfo newCustomerBaseInfo = new CustomerBaseInfo();
                newCustomerBaseInfo.setName(dto.getName());
                newCustomerBaseInfo.setMemberId(dto.getMemberId());
                newCustomerBaseInfo.setGroupId(customer.getGroupId());
                newCustomerBaseInfo.setYn(true);
                newCustomerBaseInfo.setIdCode(member.getCredentNo());
                newCustomerBaseInfo.setMobile(member.getPhone());
                newCustomerBaseInfo.setContact(member.getRealName());
                newCustomerBaseInfo.setCusType(CusTypeEnum.ofValue(dto.getCusType()));
                customerBaseInfoService.save(newCustomerBaseInfo);
                customer.setBaseId(newCustomerBaseInfo.getId());
            }
            customerService.updateById(customer);
        });
    }


    public Page<MemberVehicleVO> page(MemberVehicleQueryVO vo) {
        PageData<MemberVehicle> page = new PageData<MemberVehicle>(vo);
        LambdaQueryWrapper<MemberVehicle> lambdaQueryWrapper = Wrappers.<MemberVehicle>lambdaQuery()
                .eq(MemberVehicle::getYn, true)
                .and(!StringUtils.isEmpty(vo.getKeywords()),
                        w1 -> w1.like(MemberVehicle::getFrameNo, vo.getKeywords())
                                .or(w2 -> w2.like(MemberVehicle::getPlateNo, vo.getKeywords()))
                                .or(w3 -> w3.like(MemberVehicle::getName, vo.getKeywords())));
        memberVehicleService.page(page, lambdaQueryWrapper);
        return PageUtils.convertPage(page, MemberVehicleVO::with).toBasePage();
    }

    @Transactional(rollbackFor = Exception.class)
    public void deleteVehicle(IdVO vo) {
        memberVehicleService.update(Wrappers.<MemberVehicle>lambdaUpdate().set(MemberVehicle::getYn, false)
                .eq(MemberVehicle::getId, vo.getId()));
    }


}
